
/************************************************************************/
/* Bentigt die I2C-Lib von Peter Fleury								*/
/************************************************************************/

void sht21_init (int resolution, int end_of_batt, int OCH, int OTP_r)
{
	int userreg;
		
	i2c_start_wait(SHT21+I2C_WRITE);
	 
	i2c_write(0b11111110); //soft reset
	
	_delay_ms(20);	//wait for the sht21 to reboot
	
	i2c_rep_start(SHT21+I2C_WRITE);
	
	i2c_write(0b11100110); //enter config register
	
	/************************************************************************/
	/* User  config register:                                               */
	/*______________________________________________________________________*/
	/* Bit(s)	| Description									| Default	*/
	/*__________|_______________________________________________|___________*/
	/* 7, 0		| Measurement resolution						| 00		*/
	/*			|	|RH bits	| T bits						|			*/
	/*			|00	| 12		| 14							|			*/
	/*			|01	| 8			| 12							|			*/
	/*			|10	| 10		| 13							|			*/
	/*			|11	| 11		| 11							|			*/
	/*__________|_______________________________________________|___________*/
	/* 6		| Status: End of battery						| 0			*/
	/*			| 0: VDD > 2.25V								|			*/
	/*			| 1: VDD < 2.25V								|			*/
	/*__________|_______________________________________________|___________*/
	/* 3,4,5	| Have no consequences/Reserved					|			*/
	/*__________|_______________________________________________|___________*/
	/* 2		| Enable On-Chip-Heater							| 0			*/
	/*__________|_______________________________________________|___________*/
	/* 1		| Disable OTP Reload							| 1			*/
	/*______________________________________________________________________*/	
	/************************************************************************/
	userreg = 0;
	
	
	/************************************************************************/
	/* Measurement resolution												*/
	/************************************************************************/
	switch (resolution)
	{
		case RH12bits_T14bits	: userreg = userreg +0;
			break;
			
		case RH8bits_T12bits	: userreg = userreg +1;
			break;
			
		case RH10bits_T13bits	: userreg = userreg +128;
			break;
			
		case RH11bits_T11bits	: userreg = userreg +129;
			break;	
	}
	
	/************************************************************************/
	/* End of Battery                                                       */
	/* The end of battery alert is activated when the battery power falls	*/
	/* below 2.25V.															*/
	/************************************************************************/
	if (end_of_batt = End_of_Batt_bigger_than_2V25)
	{
		userreg = userreg +0;
	}
	else //End_of_Batt_smaller_than_2V25
	{
		userreg = userreg +64;
	}	
	
	/************************************************************************/
	/* On Chip Heater                                                       */
	/* The heater is intended to be used for functionality diagnosis 		*/
	/* relative humidity drops upon rising temperature. The heater consumes */
	/* about 5.5mW and provides a temperature increase of about 0.5  1.5C.*/
	/************************************************************************/
	if ( OCH == On_Chip_Heater)
	{
		userreg = userreg +4;
	} 
	else //No_On_Chip_Heater
	{
		userreg = userreg +0;
	}
	
	/************************************************************************/
	/* OTP Reload                                                           */
	/* OTP Reload is a safety feature and loads the entire OTP settings to	*/
	/* the register, with the exception of the heater bit, before every		*/
	/* measurement. This feature is disabled per default and is not			*/
	/* recommended for use. Please use Soft Reset instead  it contains OTP	*/
	/* Reload.																*/
	/************************************************************************/
	if ( OTP_r == OTP_Reload)
	{
		userreg = userreg +2;
	} 
	else //No_OTP_Reload
	{
		userreg = userreg +0;
	}		
	 
	i2c_write(userreg); //user reg write
	
	i2c_stop();	
}

/************************************************************************/

int convert_sht21_str_sht21_val (int data, int type)
{
	data.0 = 0;
	data.1 = 0;
	
	if ( type == TEMPMODE )
	{
		//conversion bcd decimal
		Stelle = 0;
		while (Stelle <16)
		{
			sht21_temp_val = 2^Stelle;
		}
		
		//conversion into real value
		/************************************************************************/
		/*				   S(RH)												*/
		/* RH = -6 + 125* -------												*/
		/*				   2^16													*/
		/*																		*/
		/* The physical value RH given above corresponds to the relative		*/ 
		/* humidity above liquid water according to World Meteorological		*/
		/* Organization (WMO).													*/
		/************************************************************************/
		
		sht21_temp_val = sht21_temp_val / 2^16;
		sht21_temp_val = sht21_temp_val * 125;
		sht21_temp_val = -6 + sht21_temp_val;
		
		return sht21_temp_val;  
	}
	else
	{
		//conversion bcd decimal
		Stelle = 0;
		while (Stelle <16)
		{
			sht21_humi_val = 2^Stelle;
		}
		
		//conversion into real value
		/************************************************************************/
		/*					     S(T)											*/
		/* T = -46.85 + 175.72* -------											*/
		/*				         2^16											*/
		/************************************************************************/
		
		sht21_humi_val = sht21_humi_val / 2^16;
		sht21_humi_val = sht21_humi_val * 175.72;
		sht21_humi_val = -46.85 + sht21_humi_val;
		
		return sht21_humi_val;
	}	
}

/************************************************************************/

int get_sht21_data_str (int measuretype, int holdmode, int crcmode)
{
	int holdmode;
	int crcmode;
	int measuretype;
	int string[8] AS command;
	int string[16] AS sht21_data;
		
	command.0 =1;
	command.1 =1;
	command.2 =1;
	command.3 =holdmode;
	command.4 = 0;
	command.5 = measuretype.0;
	command.6 = measuretype.1;
	command.7 = measuretype.3
	
	i2c_start_wait(SHT21+I2C_WRITE);	
	i2c_write(command);	
	
	if (holdmode == HOLD_MASTER)
	{
		while(SCL = low);; // wait until measuring and conversion complete
		
		i2c_rep_start(SHT21+I2C_WRITE);
	
		sht21_data.0 = i2c_readAck(); //read MSB
	
		if (crcmode = WITH_CRC)
		{
			sht21_data.8 = i2c_readNak(); //read LSB //Nack to omit CRC transmission
		
			i2c_stop();
			return sht21_data;
		}
		else
		{
			sht21_data.8 = i2c_readAck(); //read LSB //Ack for CRC transmission
			sht21_crc = i2c_readNak(); //read crc
	
			i2c_stop();
			return sht21_data, sht21_crc;
		}		
	}
	else  //NO_HOLD_MASTER
	{	
		while (i2c_rep_start(SHT21+I2C_WRITE) == NACK)
		{
			i2c_rep_start(SHT21+I2C_WRITE);
		}
		
		sht21_data.0 = i2c_readAck(); //read MSB
	
		if (crcmode = WITH_CRC)
		{
			sht21_data.8 = i2c_readNak(); //read LSB //Nack to omit CRC transmission
		
			i2c_stop();
			return sht21_data;
		}
		else
		{
			sht21_data.8 = i2c_readAck(); //read LSB //Ack for CRC transmission
			sht21_crc = i2c_readNak(); //read crc
	
			i2c_stop();
			return sht21_data, sht21_crc;
		}		
	}		
}

/************************************************************************/

void reset_sht21 (void)
{
	i2c_start(SHT21+I2C_WRITE);
	i2c_write(0b11111110); //soft reset
	_delay_ms(20) //wait for the sht21 to reboot
	i2c_stop();
}